Option Explicit On
Option Strict On

#Region " Imports "
Imports Mono.Cecil
Imports Mono.Cecil.Cil
Imports Reflector.CodeModel
Imports Reflexil.Utils
Imports System.Windows.Forms
#End Region

Namespace Reflexil.Forms

    Friend Class GenericMemberReferenceForm(Of T As MemberReference)
        Implements IReflectionVisitor
        Implements IComparer

#Region " Constants "
        Const EXPANDER_NODE_KEY As String = "|-expander-|"
#End Region

#Region " Fields "
        Private m_selected As T
        Private m_nodes As New Dictionary(Of Object, TreeNode)
        Private m_visiteditems As New Dictionary(Of IReflectionVisitable, IReflectionVisitable)
        Private m_orders As New Dictionary(Of Type, Integer)
#End Region

#Region " Properties "
        Public ReadOnly Property SelectedItem() As MemberReference
            Get
                Return m_selected
            End Get
        End Property
#End Region

#Region " Events "
        Private Sub TreeView_BeforeExpand(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewCancelEventArgs) Handles TreeView.BeforeExpand
            LoadNodeOnDemand(e.Node)
        End Sub

        Private Sub TreeView_NodeMouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeNodeMouseClickEventArgs) Handles TreeView.NodeMouseClick
            If Not IsNothing(e.Node.Tag) Then
                If GetType(T).IsAssignableFrom(e.Node.Tag.GetType) Then
                    m_selected = DirectCast(e.Node.Tag, T)
                    ButOk.Enabled = True
                Else
                    ButOk.Enabled = False
                End If
            Else
                ButOk.Enabled = False
            End If
        End Sub

        Private Sub MemberReferenceForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
            TreeView.Focus()
        End Sub
#End Region

#Region " Methods "
        Public Sub New(ByVal selected As T)
            InitializeComponent()

            Text = Text & GetType(T).Name.Replace("Reference", String.Empty).ToLower
            ImageList.Images.AddStrip(DataManager.GetInstance.GetAllImages)

            For Each asm As IAssembly In DataManager.GetInstance.GetReflectorAssemblies
                AppendRootNode(asm)
            Next

            m_orders.Add(GetType(AssemblyDefinition), 0)
            m_orders.Add(GetType(TypeDefinition), 1)
            m_orders.Add(GetType(MethodDefinition), 2)
            m_orders.Add(GetType(PropertyDefinition), 3)
            m_orders.Add(GetType(EventDefinition), 3)
            m_orders.Add(GetType(FieldDefinition), 5)

            TreeView.TreeViewNodeSorter = Me

            ButOk.Enabled = Not IsNothing(selected) AndAlso SelectItem(selected)
        End Sub

#Region " Selection "
        Public Function GetAssemblyDefinitionByNodeName(ByVal name As String) As AssemblyDefinition
            For Each subNode As TreeNode In TreeView.Nodes
                If subNode.Text = name Then
                    If TypeOf (subNode.Tag) Is IAssembly Then
                        LoadNodeOnDemand(subNode)
                    End If
                    Return DirectCast(subNode.Tag, AssemblyDefinition)
                End If
            Next
            Return Nothing
        End Function

        Public Function StripGenerics(ByVal item As TypeReference, ByVal str As String) As String
            If TypeOf (item) Is GenericInstanceType Then
                For Each arg As TypeReference In DirectCast(item, GenericInstanceType).GenericArguments
                    str = str.Replace(String.Format("<{0}>", arg.FullName), String.Empty)
                Next
            End If
            Return str
        End Function

        Public Function GetTypeDefinition(ByVal item As TypeReference) As TypeDefinition
            Dim moddef As ModuleDefinition = Nothing

            If TypeOf (item.Scope) Is ModuleDefinition Then
                moddef = DirectCast(item.Scope, ModuleDefinition)
                GetAssemblyDefinitionByNodeName(moddef.Assembly.Name.Name)
            ElseIf TypeOf (item.Scope) Is AssemblyNameReference Then
                Dim anr As AssemblyNameReference = DirectCast(item.Scope, AssemblyNameReference)
                Dim asmdef As AssemblyDefinition = GetAssemblyDefinitionByNodeName(anr.Name)
                If Not IsNothing(asmdef) Then
                    moddef = asmdef.MainModule
                End If
            End If

            If Not IsNothing(moddef) Then
                Dim typedef As TypeDefinition = moddef.Types(StripGenerics(item, item.FullName))

                If Not IsNothing(typedef) Then
                    If Not IsNothing(typedef.DeclaringType) Then
                        GetTypeDefinition(typedef.DeclaringType)
                    End If
                    LoadNodeOnDemand(m_nodes(moddef))
                    LoadNodeOnDemand(m_nodes(typedef))
                    Return typedef
                End If
            End If

            Return Nothing
        End Function

        Public Function GetMethodDefinition(ByVal item As MethodReference) As MethodDefinition
            Dim typedef As TypeDefinition = GetTypeDefinition(item.DeclaringType)
            If Not IsNothing(typedef) Then
                Dim methods As New ArrayList
                methods.AddRange(typedef.Constructors)
                methods.AddRange(typedef.Methods)
                For Each method As MethodDefinition In methods
                    If StripGenerics(typedef, method.ToString) = StripGenerics(item.DeclaringType, item.ToString) Then
                        Return method
                    End If
                Next
            End If
            Return Nothing
        End Function

        Public Function GetFieldDefinition(ByVal item As FieldReference) As FieldDefinition
            Dim typedef As TypeDefinition = GetTypeDefinition(item.DeclaringType)
            If Not IsNothing(typedef) Then
                For Each field As FieldDefinition In typedef.Fields
                    If StripGenerics(typedef, field.ToString) = StripGenerics(item.DeclaringType, item.ToString) Then
                        Return field
                    End If
                Next
            End If
            Return Nothing
        End Function

        Public Function SelectItem(ByVal item As MemberReference) As Boolean
            Dim itemtag As Object = Nothing

            If TypeOf (item) Is TypeReference Then
                itemtag = GetTypeDefinition(DirectCast(item, TypeReference))
            ElseIf TypeOf (item) Is MethodReference Then
                itemtag = GetMethodDefinition(DirectCast(item, MethodReference))
            ElseIf TypeOf (item) Is FieldReference Then
                itemtag = GetFieldDefinition(DirectCast(item, FieldReference))
            End If

            If Not IsNothing(itemtag) AndAlso m_nodes.ContainsKey(itemtag) Then
                TreeView.SelectedNode = m_nodes(itemtag)
                m_selected = DirectCast(item, T)
                Return True
            End If

            Return False
        End Function
#End Region

#Region " Cosmetic "
        Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements System.Collections.IComparer.Compare
            Dim xn As TreeNode = DirectCast(x, TreeNode)
            Dim yn As TreeNode = DirectCast(y, TreeNode)

            Dim result As Integer = 0
            If Not IsNothing(xn.Tag) AndAlso Not IsNothing(yn.Tag) Then
                If m_orders.ContainsKey(xn.Tag.GetType) Then
                    result = m_orders(xn.Tag.GetType).CompareTo(m_orders(yn.Tag.GetType))
                End If
            End If
            If result = 0 Then
                result = xn.Text.CompareTo(yn.Text)
            End If
            Return result
        End Function

        Private Function ScopeOffset(ByVal attributes As Integer, ByVal mask As Integer, ByVal publicValue As Integer, ByVal friendValue As Integer, ByVal protectedFriendValue As Integer, ByVal protectedValue As Integer, ByVal privateValue As Integer) As Integer
            Dim scopes() As Integer = {publicValue, friendValue, protectedFriendValue, protectedValue, privateValue}
            attributes = attributes And mask

            For i As Integer = 0 To scopes.Length - 1
                If attributes = scopes(i) Then
                    Return i
                End If
            Next

            Return 0
        End Function

        Private Function ImageIndex(ByVal obj As Object) As EReflectorImages
            Dim offset As EReflectorImages = EReflectorImages.Empty

            If TypeOf (obj) Is TypeDefinition Then
                Dim typedef As TypeDefinition = DirectCast(obj, TypeDefinition)
                If typedef.IsEnum Then
                    offset = EReflectorImages.PublicEnum
                ElseIf typedef.IsInterface Then
                    offset = EReflectorImages.PublicInterface
                ElseIf typedef.IsValueType Then
                    offset = EReflectorImages.PublicStructure
                Else
                    offset = EReflectorImages.PublicClass
                End If
                If (typedef.Attributes And TypeAttributes.VisibilityMask) < TypeAttributes.Public Then
                    offset = DirectCast(offset + EReflectorImages.FriendClass - EReflectorImages.PublicClass, EReflectorImages)
                Else
                    offset = DirectCast(offset + ScopeOffset(CInt(typedef.Attributes), CInt(TypeAttributes.VisibilityMask), CInt(TypeAttributes.NestedPublic), CInt(TypeAttributes.NestedAssembly), CInt(TypeAttributes.NestedFamORAssem), CInt(TypeAttributes.NestedFamily), CInt(TypeAttributes.NestedPrivate)), EReflectorImages)
                End If
            ElseIf TypeOf (obj) Is PropertyDefinition Then
                Dim propdef As PropertyDefinition = DirectCast(obj, PropertyDefinition)
                If IsNothing(propdef.GetMethod) Then
                    If propdef.SetMethod.IsStatic Then
                        offset = EReflectorImages.PublicSharedWriteOnlyProperty
                    Else
                        offset = EReflectorImages.PublicWriteOnlyProperty
                    End If
                ElseIf IsNothing(propdef.SetMethod) Then
                    If propdef.GetMethod.IsStatic Then
                        offset = EReflectorImages.PublicSharedReadOnlyProperty
                    Else
                        offset = EReflectorImages.PublicReadOnlyProperty
                    End If
                Else
                    If propdef.GetMethod.IsStatic Then
                        offset = EReflectorImages.PublicSharedProperty
                    Else
                        offset = EReflectorImages.PublicProperty
                    End If
                End If
            ElseIf TypeOf (obj) Is MethodDefinition Then
                Dim metdef As MethodDefinition = DirectCast(obj, MethodDefinition)
                If metdef.IsConstructor Then
                    If metdef.IsStatic Then
                        offset = EReflectorImages.PublicSharedConstructor
                    Else
                        offset = EReflectorImages.PublicConstructor
                    End If
                Else
                    If metdef.IsVirtual Then
                        If metdef.IsStatic Then
                            offset = EReflectorImages.PublicSharedOverrideMethod
                        Else
                            offset = EReflectorImages.PublicOverrideMethod
                        End If
                    Else
                        If metdef.IsStatic Then
                            offset = EReflectorImages.PublicSharedMethod
                        Else
                            offset = EReflectorImages.PublicMethod
                        End If
                    End If
                End If
                offset = DirectCast(offset + ScopeOffset(metdef.Attributes, MethodAttributes.MemberAccessMask, MethodAttributes.Public, MethodAttributes.Assem, MethodAttributes.FamORAssem, MethodAttributes.Family, MethodAttributes.Private), EReflectorImages)
            ElseIf TypeOf (obj) Is FieldDefinition Then
                Dim field As FieldDefinition = DirectCast(obj, FieldDefinition)
                If field.IsLiteral AndAlso field.IsStatic Then
                    offset = EReflectorImages.PublicEnumValue
                Else
                    If field.IsStatic Then
                        offset = EReflectorImages.PublicSharedField
                    Else
                        offset = EReflectorImages.PublicField
                    End If
                End If
                offset = DirectCast(offset + ScopeOffset(field.Attributes, FieldAttributes.FieldAccessMask, FieldAttributes.Public, FieldAttributes.Assembly, FieldAttributes.FamORAssem, FieldAttributes.Family, FieldAttributes.Private), EReflectorImages)
            ElseIf TypeOf (obj) Is ModuleDefinition Then
                offset = EReflectorImages.Module
            ElseIf TypeOf (obj) Is EventDefinition Then
                Dim evtdef As EventDefinition = DirectCast(obj, EventDefinition)
                If evtdef.AddMethod.IsStatic Then
                    offset = EReflectorImages.PublicSharedEvent
                Else
                    offset = EReflectorImages.PublicEvent
                End If
            ElseIf TypeOf (obj) Is AssemblyDefinition Or TypeOf (obj) Is IAssembly Then
                offset = EReflectorImages.Assembly
            ElseIf TypeOf (obj) Is String Then
                offset = EReflectorImages.PublicNamespace
            End If

            Return offset
        End Function

        Public Function DisplayString(ByVal obj As Object) As String
            If TypeOf (obj) Is MethodDefinition Then
                Dim metdef As MethodDefinition = DirectCast(obj, MethodDefinition)
                Return metdef.ToString.Substring(metdef.ToString.IndexOf("::") + 2) + " : " & metdef.ReturnType.ReturnType.ToString
            ElseIf TypeOf (obj) Is PropertyDefinition Then
                Dim propdef As PropertyDefinition = DirectCast(obj, PropertyDefinition)
                Return propdef.ToString.Substring(propdef.ToString.IndexOf("::") + 2) + " : " & propdef.PropertyType.ToString
            ElseIf TypeOf (obj) Is FieldDefinition Then
                Dim flddef As FieldDefinition = DirectCast(obj, FieldDefinition)
                Return flddef.ToString.Substring(flddef.ToString.IndexOf("::") + 2) + " : " & flddef.FieldType.ToString
            ElseIf TypeOf (obj) Is ModuleDefinition Then
                Dim moddef As ModuleDefinition = DirectCast(obj, ModuleDefinition)
                Return moddef.Name
            ElseIf TypeOf (obj) Is TypeDefinition Then
                Dim typedef As TypeDefinition = DirectCast(obj, TypeDefinition)
                Return typedef.Name
            ElseIf TypeOf (obj) Is AssemblyDefinition Then
                Dim asmdef As AssemblyDefinition = DirectCast(obj, AssemblyDefinition)
                Return asmdef.Name.Name
            ElseIf TypeOf (obj) Is IAssembly Then
                Dim iasm As IAssembly = DirectCast(obj, IAssembly)
                Return iasm.Name
            ElseIf TypeOf (obj) Is EventDefinition Then
                Dim evtdef As EventDefinition = DirectCast(obj, EventDefinition)
                Return evtdef.Name
            Else
                Return obj.ToString
            End If
        End Function
#End Region

#Region " Node management "
        Private Sub LoadNodeOnDemand(ByVal node As TreeNode)
            If node.Nodes.ContainsKey(EXPANDER_NODE_KEY) Then
                node.Nodes.Remove(node.Nodes(EXPANDER_NODE_KEY))
            End If
            If TypeOf (node.Tag) Is IReflectionVisitable Then
                Dim visitable As IReflectionVisitable = DirectCast(node.Tag, IReflectionVisitable)
                If Not m_visiteditems.ContainsKey(visitable) Then
                    visitable.Accept(Me)
                    m_visiteditems.Add(visitable, visitable)
                End If
            ElseIf TypeOf (node.Tag) Is IAssembly Then
                Dim iasm As IAssembly = DirectCast(node.Tag, IAssembly)
                Dim asmdef As AssemblyDefinition = DataManager.GetInstance.GetAssemblyDefinition(iasm.Location)

                m_nodes.Remove(node.Tag)
                m_nodes.Add(asmdef, node)
                node.Tag = asmdef

                For Each moddef As ModuleDefinition In asmdef.Modules
                    AppendNode(asmdef, moddef, moddef.Types.Count > 0)
                Next
            End If
        End Sub

        Private Sub AppendRootNode(ByVal root As Object)
            Dim rootnode As New TreeNode(DisplayString(root))
            rootnode.ImageIndex = ImageIndex(root)
            rootnode.SelectedImageIndex = rootnode.ImageIndex
            rootnode.Tag = root
            rootnode.Nodes.Add(EXPANDER_NODE_KEY, EXPANDER_NODE_KEY)
            TreeView.Nodes.Add(rootnode)
            m_nodes.Add(root, rootnode)
        End Sub

        Private Sub AppendNode(ByVal ownernode As TreeNode, ByVal child As Object, ByVal createExpander As Boolean)
            If Not m_nodes.ContainsKey(child) Then
                Dim childnode As New TreeNode(DisplayString(child))
                childnode.ImageIndex = ImageIndex(child)
                childnode.SelectedImageIndex = childnode.ImageIndex
                childnode.Tag = child
                If createExpander Then
                    childnode.Nodes.Add(EXPANDER_NODE_KEY, EXPANDER_NODE_KEY)
                End If
                ownernode.Nodes.Add(childnode)
                m_nodes.Add(child, childnode)
            End If
        End Sub

        Private Sub AppendNode(ByVal owner As Object, ByVal child As Object, ByVal createExpander As Boolean)
            Dim ownernode As TreeNode = m_nodes(owner)
            AppendNode(ownernode, child, createExpander)
        End Sub
#End Region

#Region " Visitor implementation "
        Public Sub VisitConstructorCollection(ByVal ctors As Mono.Cecil.ConstructorCollection) Implements Mono.Cecil.IReflectionVisitor.VisitConstructorCollection
            For Each constructor As MethodDefinition In ctors
                AppendNode(constructor.DeclaringType, constructor, False)
            Next
        End Sub

        Public Sub VisitEventDefinitionCollection(ByVal events As Mono.Cecil.EventDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitEventDefinitionCollection
            For Each evt As EventDefinition In events
                AppendNode(evt.DeclaringType, evt, True)
                If Not IsNothing(evt.AddMethod) Then
                    AppendNode(evt, evt.AddMethod, False)
                End If
                If Not IsNothing(evt.RemoveMethod) Then
                    AppendNode(evt, evt.RemoveMethod, False)
                End If
            Next
        End Sub

        Public Sub VisitFieldDefinitionCollection(ByVal fields As Mono.Cecil.FieldDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitFieldDefinitionCollection
            For Each field As FieldDefinition In fields
                AppendNode(field.DeclaringType, field, False)
            Next
        End Sub

        Public Sub VisitMethodDefinitionCollection(ByVal methods As Mono.Cecil.MethodDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitMethodDefinitionCollection
            For Each method As MethodDefinition In methods
                If Not method.IsSpecialName OrElse method.IsConstructor Then
                    AppendNode(method.DeclaringType, method, False)
                End If
            Next
        End Sub

        Public Sub VisitNestedTypeCollection(ByVal nestedTypes As Mono.Cecil.NestedTypeCollection) Implements Mono.Cecil.IReflectionVisitor.VisitNestedTypeCollection
            For Each nestedType As TypeDefinition In nestedTypes
                AppendNode(nestedType.DeclaringType, nestedType, True)
            Next
        End Sub

        Public Sub VisitPropertyDefinitionCollection(ByVal properties As Mono.Cecil.PropertyDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitPropertyDefinitionCollection
            For Each [property] As PropertyDefinition In properties
                AppendNode([property].DeclaringType, [property], True)
                If Not IsNothing([property].GetMethod) Then
                    AppendNode([property], [property].GetMethod, False)
                End If
                If Not IsNothing([property].SetMethod) Then
                    AppendNode([property], [property].SetMethod, False)
                End If
            Next
        End Sub

        Public Sub VisitTypeDefinitionCollection(ByVal types As Mono.Cecil.TypeDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitTypeDefinitionCollection
            For Each type As TypeDefinition In types
                If (type.Attributes And TypeAttributes.VisibilityMask) <= TypeAttributes.Public Then
                    AppendNode(type.Module, type.Namespace, True)
                    AppendNode(type.Namespace, type, True)
                End If
            Next
        End Sub
#End Region

#Region " Unimplemented vistor "
        Public Sub VisitEventDefinition(ByVal evt As Mono.Cecil.EventDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitEventDefinition
        End Sub

        Public Sub VisitFieldDefinition(ByVal field As Mono.Cecil.FieldDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitFieldDefinition
        End Sub

        Public Sub VisitModuleDefinition(ByVal [module] As Mono.Cecil.ModuleDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitModuleDefinition
        End Sub

        Public Sub VisitNestedType(ByVal nestedType As Mono.Cecil.TypeDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitNestedType
        End Sub

        Public Sub VisitPropertyDefinition(ByVal [property] As Mono.Cecil.PropertyDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitPropertyDefinition
        End Sub

        Public Sub VisitTypeDefinition(ByVal type As Mono.Cecil.TypeDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitTypeDefinition
        End Sub

        Public Sub VisitConstructor(ByVal ctor As Mono.Cecil.MethodDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitConstructor
        End Sub

        Public Sub VisitMethodDefinition(ByVal method As Mono.Cecil.MethodDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitMethodDefinition
        End Sub

        Public Sub TerminateModuleDefinition(ByVal [module] As Mono.Cecil.ModuleDefinition) Implements Mono.Cecil.IReflectionVisitor.TerminateModuleDefinition
        End Sub

        Public Sub VisitExternType(ByVal externType As Mono.Cecil.TypeReference) Implements Mono.Cecil.IReflectionVisitor.VisitExternType
        End Sub

        Public Sub VisitExternTypeCollection(ByVal externs As Mono.Cecil.ExternTypeCollection) Implements Mono.Cecil.IReflectionVisitor.VisitExternTypeCollection
        End Sub

        Public Sub VisitInterface(ByVal interf As Mono.Cecil.TypeReference) Implements Mono.Cecil.IReflectionVisitor.VisitInterface
        End Sub

        Public Sub VisitInterfaceCollection(ByVal interfaces As Mono.Cecil.InterfaceCollection) Implements Mono.Cecil.IReflectionVisitor.VisitInterfaceCollection
        End Sub

        Public Sub VisitMemberReference(ByVal member As Mono.Cecil.MemberReference) Implements Mono.Cecil.IReflectionVisitor.VisitMemberReference
        End Sub

        Public Sub VisitMemberReferenceCollection(ByVal members As Mono.Cecil.MemberReferenceCollection) Implements Mono.Cecil.IReflectionVisitor.VisitMemberReferenceCollection
        End Sub

        Public Sub VisitCustomAttribute(ByVal customAttr As Mono.Cecil.CustomAttribute) Implements Mono.Cecil.IReflectionVisitor.VisitCustomAttribute
        End Sub

        Public Sub VisitCustomAttributeCollection(ByVal customAttrs As Mono.Cecil.CustomAttributeCollection) Implements Mono.Cecil.IReflectionVisitor.VisitCustomAttributeCollection
        End Sub

        Public Sub VisitGenericParameter(ByVal genparam As Mono.Cecil.GenericParameter) Implements Mono.Cecil.IReflectionVisitor.VisitGenericParameter
        End Sub

        Public Sub VisitGenericParameterCollection(ByVal genparams As Mono.Cecil.GenericParameterCollection) Implements Mono.Cecil.IReflectionVisitor.VisitGenericParameterCollection
        End Sub

        Public Sub VisitMarshalSpec(ByVal marshalSpec As Mono.Cecil.MarshalSpec) Implements Mono.Cecil.IReflectionVisitor.VisitMarshalSpec
        End Sub

        Public Sub VisitSecurityDeclaration(ByVal secDecl As Mono.Cecil.SecurityDeclaration) Implements Mono.Cecil.IReflectionVisitor.VisitSecurityDeclaration
        End Sub

        Public Sub VisitSecurityDeclarationCollection(ByVal secDecls As Mono.Cecil.SecurityDeclarationCollection) Implements Mono.Cecil.IReflectionVisitor.VisitSecurityDeclarationCollection
        End Sub

        Public Sub VisitTypeReference(ByVal type As Mono.Cecil.TypeReference) Implements Mono.Cecil.IReflectionVisitor.VisitTypeReference
        End Sub

        Public Sub VisitTypeReferenceCollection(ByVal refs As Mono.Cecil.TypeReferenceCollection) Implements Mono.Cecil.IReflectionVisitor.VisitTypeReferenceCollection
        End Sub

        Public Sub VisitOverride(ByVal ov As Mono.Cecil.MethodReference) Implements Mono.Cecil.IReflectionVisitor.VisitOverride
        End Sub

        Public Sub VisitOverrideCollection(ByVal meth As Mono.Cecil.OverrideCollection) Implements Mono.Cecil.IReflectionVisitor.VisitOverrideCollection
        End Sub

        Public Sub VisitParameterDefinition(ByVal parameter As Mono.Cecil.ParameterDefinition) Implements Mono.Cecil.IReflectionVisitor.VisitParameterDefinition
        End Sub

        Public Sub VisitParameterDefinitionCollection(ByVal parameters As Mono.Cecil.ParameterDefinitionCollection) Implements Mono.Cecil.IReflectionVisitor.VisitParameterDefinitionCollection
        End Sub

        Public Sub VisitPInvokeInfo(ByVal pinvk As Mono.Cecil.PInvokeInfo) Implements Mono.Cecil.IReflectionVisitor.VisitPInvokeInfo
        End Sub
#End Region

#End Region

    End Class

End Namespace

